
(* SPAN - A Security Protocol Animator
   Copyright (C) 2006-2008 Yann Glouche, Thomas Genet and Erwan Houssay

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.5. Last modification 23/09/08
*)
(* interpretation du resultat de la compilation d'une specification HLPSL*)

(* specification HLPSL analysee : protocole EKE *)
let hlpsl_file = "/udd/ehoussay/dev/span/testsuite/hlpsl/EKE.hlpsl";;

(* charge les modules *)

#load "types.cmo";;
#load "globals.cmo";;
#load "error_handler.cmo";;
#load "lexer.cmo";;
#load "utils.cmo";;
#load "interface.cmo";;
#load "utils_debug.cmo";;
#load "semantic_check.cmo";;
#load "parser.cmo";;
#load "section_rules.cmo";;
#load "instantiation.cmo";;

(* initialise le tampon de lecture *)

Globals.input_file := hlpsl_file;;
let input_channel =
  if (!Globals.input_file = "") then 
    (Globals.flag_stdout := true;
     Globals.flag_split := false;
     stdin)
  else
    try (
      if not !Globals.flag_stdout then
	(if (Filename.check_suffix !Globals.input_file ".hlpsl") then
	   Globals.output_file := Filename.basename (Filename.chop_suffix !Globals.input_file ".hlpsl")
      	 else
	   Globals.output_file := Filename.basename !Globals.input_file;
	 if (!Globals.output_dir = "") then
	   Globals.output_dir := (Filename.dirname !Globals.input_file)^"/";
	 Globals.tmp_file := (!Globals.output_dir)^(!Globals.output_file)^".if");
      open_in !Globals.input_file)
    with Sys_error(f) ->
      Error_handler.handleSystemError(f);;
Globals.lexbuf := Lexing.from_channel input_channel;;

(* compile les specifications HLPSL *)

let specif = Parser.specification Lexer.token !Globals.lexbuf;;
(*>>
val specif : Types.specification_class = <obj>
*)

(* ************************* *)
(* Types.specification_class *)
(* Elements de plus haut niveau dans la specification formelle *)

(* roles definis dans la specification formelle *)

specif#get_roles;;
(*>>
- : Types.role_type list =
[Types.Basic <obj>; Types.Basic <obj>; Types.Composition <obj>; Types.Composition <obj>]
*)
let [Types.Basic basic1; Types.Basic basic2; Types.Composition compo3; Types.Composition compo4] = specif#get_roles;;
(*>>
val basic1 : Types.basic_hlpsl_role = <obj>
val basic2 : Types.basic_hlpsl_role = <obj>
val compo3 : Types.composition_role = <obj>
val compo4 : Types.composition_role = <obj>
*)

(* objectifs de verification *)

specif#get_goals;;
(*>>
- : Types.goals_type list = [Types.Secrecy [15; 38]; Types.Authentication [25]; Types.Authentication [22]]
*)

(* appel de role principal *)

specif#get_main_role;;
(*>>
- : Types.composition_type = Types.Call <obj>
*)
let Types.Call main_call = specif#get_main_role;;
(*>>
val main_call : Types.call_type = <obj>
*)

(* ************************ *)
(* Types.generic_hlpsl_role *)
(* Informations definies les roles de maniere generique *)

(* nom du role defini *)

let name1 = basic1#get_name;;
let name2 = basic2#get_name;;
let name3 = compo3#get_name;;
let name4 = compo4#get_name;;
(*>>
val name1 : int = 4
val name2 : int = 27
val name3 : int = 40
val name4 : int = 48
*)

(* parametres formels du role *)

basic1#get_param;;
basic2#get_param;;
compo3#get_param;;
compo4#get_param;;
(*>>
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = []
*)
let [param11;param12;param13;param14;param15] = basic1#get_param;;
let [param21;param22;param23;param24;param25] = basic1#get_param;;
let [param31;param32;param33] = compo3#get_param;;
(*>>
val param11 : Types.hlpsl_variable = <obj>
val param12 : Types.hlpsl_variable = <obj>
val param13 : Types.hlpsl_variable = <obj>
val param14 : Types.hlpsl_variable = <obj>
val param15 : Types.hlpsl_variable = <obj>
val param21 : Types.hlpsl_variable = <obj>
val param22 : Types.hlpsl_variable = <obj>
val param23 : Types.hlpsl_variable = <obj>
val param24 : Types.hlpsl_variable = <obj>
val param25 : Types.hlpsl_variable = <obj>
val param31 : Types.hlpsl_variable = <obj>
val param32 : Types.hlpsl_variable = <obj>
val param33 : Types.hlpsl_variable = <obj>
*)

(* The list of variables in the owned declaration *)

basic1#get_owned;;
basic2#get_owned;;
compo3#get_owned;;
compo4#get_owned;;
(*>>
- : Types.hlpsl_variable list = []
- : Types.hlpsl_variable list = []
- : Types.hlpsl_variable list = []
- : Types.hlpsl_variable list = []
*)

(* variables locales du role *)

basic1#get_local;;
basic2#get_local;;
compo3#get_local;;
compo4#get_local;;
(*>>
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>; <obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>; <obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = [<obj>; <obj>; <obj>; <obj>]
- : Types.hlpsl_variable list = []
*)
let [local11;local12;local13;local14;local15;local16] = basic1#get_local;;
let [local21;local22;local23;local24;local25;local26] = basic2#get_local;;
let [local31;local32;local33;local34] = compo3#get_local;;
(*>>
val local11 : Types.hlpsl_variable = <obj>
val local12 : Types.hlpsl_variable = <obj>
val local13 : Types.hlpsl_variable = <obj>
val local14 : Types.hlpsl_variable = <obj>
val local15 : Types.hlpsl_variable = <obj>
val local16 : Types.hlpsl_variable = <obj>
val local21 : Types.hlpsl_variable = <obj>
val local22 : Types.hlpsl_variable = <obj>
val local23 : Types.hlpsl_variable = <obj>
val local24 : Types.hlpsl_variable = <obj>
val local25 : Types.hlpsl_variable = <obj>
val local26 : Types.hlpsl_variable = <obj>
val local31 : Types.hlpsl_variable = <obj>
val local32 : Types.hlpsl_variable = <obj>
val local33 : Types.hlpsl_variable = <obj>
val local34 : Types.hlpsl_variable = <obj>
*)

(* etat initial *)

basic1#get_init;;
basic2#get_init;;
compo3#get_init;;
compo4#get_init;;
(*>>
- : Types.predicate list = [Types.Equal (Types.Base (Types.Var 10), Types.Base (Types.Const 16)); Types.Equal (Types.Base (Types.Var 20), Types.Set [])]
- : Types.predicate list = [Types.Equal (Types.Base (Types.Var 33), Types.Base (Types.Const 16)); Types.Equal (Types.Base (Types.Var 39), Types.Set [])]
- : Types.predicate list = []
- : Types.predicate list = []
*)

(* etat d'acceptation *)

basic1#get_accept;;
basic2#get_accept;;
compo3#get_accept;;
compo4#get_accept;;
(*>>
- : Types.predicate list = []
- : Types.predicate list = []
- : Types.predicate list = []
- : Types.predicate list = []
*)

(* connaissances de l'intrus *)

basic1#get_knowledge;;
basic2#get_knowledge;;
compo3#get_knowledge;;
compo4#get_knowledge;;
(*>>
- : Types.term list = []
- : Types.term list = []
- : Types.term list = []
- : Types.term list = [Types.Base (Types.Const 50); Types.Base (Types.Const 49)]
*)

(* ********************** *)
(* Types.basic_hlpsl_role *)
(* Informations definies dans les roles de base *)

(* agent jouant le role *)

let player1 = basic1#get_player;;
let player2 = basic2#get_player;;
(*>>
val player1 : int = 6
val player2 : int = 29
*)

(* transitions des roles de base (true~action ; false~reaction) *)

basic1#get_actions;;
basic2#get_actions;;
(*>>
- : (bool * Types.hlpsl_transition) list = [(false, <obj>); (false, <obj>); (false, <obj>)]
- : (bool * Types.hlpsl_transition) list = [(false, <obj>); (false, <obj>); (false, <obj>)]
*)
let [(_,action11);(_,action12);(_,action13)] = basic1#get_actions;;
let [(_,action21);(_,action22);(_,action23)] = basic2#get_actions;;
(*>>
val action11 : Types.hlpsl_transition = <obj>
val action12 : Types.hlpsl_transition = <obj>
val action13 : Types.hlpsl_transition = <obj>
val action21 : Types.hlpsl_transition = <obj>
val action22 : Types.hlpsl_transition = <obj>
val action23 : Types.hlpsl_transition = <obj>
*)

(* etiquettes identifiant les transitions *)

let label11 = action11#get_label;;
let label12 = action12#get_label;;
let label13 = action13#get_label;;
let label21 = action21#get_label;;
let label22 = action22#get_label;;
let label23 = action23#get_label;;
(*>>
val label11 : int = 17
val label12 : int = 19
val label13 : int = 24
val label21 : int = 17
val label22 : int = 19
val label23 : int = 24
*)

(* parties gauches et droites des transitions *)

action11#get_lhs;;
action11#get_rhs;;
action12#get_lhs;;
action12#get_rhs;;
action13#get_lhs;;
action13#get_rhs;;
action21#get_lhs;;
action21#get_rhs;;
action22#get_lhs;;
action22#get_rhs;;
action23#get_lhs;;
action23#get_rhs;;
(*>>
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 10), Types.Base (Types.Const 16));
 Types.Event (Types.Var 8, [Types.Base (Types.Const 18)])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 10), Types.Base (Types.Const 17));
 Types.New 11;
 Types.Event (Types.Var 9,
  [Types.Scrypt (Types.Prime (Types.Var 11), Types.Base (Types.Var 7))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 10), Types.Base (Types.Const 17));
 Types.Event (Types.Var 8,
  [Types.Scrypt
    (Types.Crypt (Types.Prime (Types.Var 12), Types.Base (Types.Var 11)),
    Types.Base (Types.Var 7))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 10), Types.Base (Types.Const 19));
 Types.New 14;
 Types.Event (Types.Var 9,
  [Types.Scrypt (Types.Prime (Types.Var 14), Types.Prime (Types.Var 12))]);
 Types.Event (Types.Const 21,
  [Types.Prime (Types.Var 12); Types.Base (Types.Const 15);
   Types.Base (Types.Var 20)]);
 Types.Event (Types.Const 23,
  [Types.Base (Types.Var 6); Types.Base (Types.Var 5);
   Types.Base (Types.Const 22); Types.Prime (Types.Var 14)]);
 Types.Equal (Types.Prime (Types.Var 20),
  Types.Cons (Types.Base (Types.Var 6),
   Types.Cons (Types.Base (Types.Var 5), Types.Base (Types.Var 20))))]
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 10), Types.Base (Types.Const 19));
 Types.Event (Types.Var 8,
  [Types.Scrypt
    (Types.Pair (Types.Base (Types.Var 14), Types.Prime (Types.Var 13)),
    Types.Base (Types.Var 12))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 10), Types.Base (Types.Const 24));
 Types.Event (Types.Var 9,
  [Types.Scrypt (Types.Prime (Types.Var 13), Types.Base (Types.Var 12))]);
 Types.Event (Types.Const 26,
  [Types.Base (Types.Var 6); Types.Base (Types.Var 5);
   Types.Base (Types.Const 25); Types.Prime (Types.Var 13)])]
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 33), Types.Base (Types.Const 16));
 Types.Event (Types.Var 31,
  [Types.Scrypt (Types.Prime (Types.Var 37), Types.Base (Types.Var 30))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 33), Types.Base (Types.Const 17));
 Types.New 34;
 Types.Event (Types.Var 32,
  [Types.Scrypt
    (Types.Crypt (Types.Prime (Types.Var 34), Types.Prime (Types.Var 37)),
    Types.Base (Types.Var 30))]);
 Types.Event (Types.Const 21,
  [Types.Prime (Types.Var 34); Types.Base (Types.Const 38);
   Types.Base (Types.Var 39)]);
 Types.Equal (Types.Prime (Types.Var 39),
  Types.Cons (Types.Base (Types.Var 28),
   Types.Cons (Types.Base (Types.Var 29), Types.Base (Types.Var 39))))]
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 33), Types.Base (Types.Const 17));
 Types.Event (Types.Var 31,
  [Types.Scrypt (Types.Prime (Types.Var 36), Types.Base (Types.Var 34))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 33), Types.Base (Types.Const 19));
 Types.New 35;
 Types.Event (Types.Var 32,
  [Types.Scrypt
    (Types.Pair (Types.Prime (Types.Var 36), Types.Prime (Types.Var 35)),
    Types.Base (Types.Var 34))]);
 Types.Event (Types.Const 23,
  [Types.Base (Types.Var 29); Types.Base (Types.Var 28);
   Types.Base (Types.Const 25); Types.Prime (Types.Var 35)])]
- : Types.hlpsl_state =
[Types.Equal (Types.Base (Types.Var 33), Types.Base (Types.Const 19));
 Types.Event (Types.Var 31,
  [Types.Scrypt (Types.Base (Types.Var 35), Types.Base (Types.Var 34))])]
- : Types.hlpsl_state =
[Types.Equal (Types.Prime (Types.Var 33), Types.Base (Types.Const 24));
 Types.Event (Types.Const 26,
  [Types.Base (Types.Var 29); Types.Base (Types.Var 28);
   Types.Base (Types.Const 22); Types.Base (Types.Var 36)])]
*)

(* ********************** *)
(* Types.composition_role *)
(* Informations definies dans les roles composes *)

(* roles appeles dans la composition *)

compo3#get_composition;;
compo4#get_composition;;
(*>>
- : Types.composition_type = Types.Parallel (Types.Call <obj>, Types.Call <obj>)
- : Types.composition_type = Types.Parallel (Types.Call <obj>, Types.Call <obj>)
*)
let Types.Parallel (Types.Call call31, Types.Call call32) = compo3#get_composition;;
let Types.Parallel (Types.Call call41, Types.Call call42) = compo4#get_composition;;
(*>>
val call31 : Types.call_type = <obj>
val call32 : Types.call_type = <obj>
val call41 : Types.call_type = <obj>
val call42 : Types.call_type = <obj>
*)

(* *************** *)
(* Types.call_type *)
(* Contenu des instances de roles appelees dans des roles composes *)

(* nom du role appele *)

let role31 = call31#get_role;;
let role32 = call32#get_role;;
let role41 = call41#get_role;;
let role42 = call42#get_role;;
let main_role = main_call#get_role;;
(*>>
val role31 : int = 4
val role32 : int = 27
val role41 : int = 40
val role42 : int = 40
val main_role : int = 48
*)

(* parametres effectifs d'appel *)

call31#get_args;;
call32#get_args;;
call41#get_args;;
call42#get_args;;
main_call#get_args;;
(*>>
- : Types.atoms_type Types.term_structure list =
[Types.Base (Types.Var 42); Types.Base (Types.Var 41);
 Types.Base (Types.Var 43); Types.Base (Types.Var 47);
 Types.Base (Types.Var 46)]
- : Types.atoms_type Types.term_structure list =
[Types.Base (Types.Var 41); Types.Base (Types.Var 42);
 Types.Base (Types.Var 43); Types.Base (Types.Var 45);
 Types.Base (Types.Var 44)]
- : Types.atoms_type Types.term_structure list =
[Types.Base (Types.Const 50); Types.Base (Types.Const 49);
 Types.Base (Types.Const 51)]
- : Types.atoms_type Types.term_structure list =
[Types.Base (Types.Const 49); Types.Base (Types.Const 50);
 Types.Base (Types.Const 51)]
- : Types.atoms_type Types.term_structure list = []
*)

(* ******************** *)
(* Types.hlpsl_variable *)
(* Parametres formels et variables definis dans les roles *)

(* identifiants des variables *)

param11#get_id;;
param12#get_id;;
param13#get_id;;
param14#get_id;;
param15#get_id;;
param21#get_id;;
param22#get_id;;
param23#get_id;;
param24#get_id;;
param25#get_id;;
param31#get_id;;
param32#get_id;;
param33#get_id;;
(*>>
- : int = 6
- : int = 5
- : int = 7
- : int = 9
- : int = 8
- : int = 6
- : int = 5
- : int = 7
- : int = 9
- : int = 8
- : int = 42
- : int = 41
- : int = 43
*)

local11#get_id;;
local12#get_id;;
local13#get_id;;
local14#get_id;;
local15#get_id;;
local16#get_id;;
local21#get_id;;
local22#get_id;;
local23#get_id;;
local24#get_id;;
local25#get_id;;
local26#get_id;;
local31#get_id;;
local32#get_id;;
local33#get_id;;
local34#get_id;;
(*>>
- : int = 10
- : int = 11
- : int = 14
- : int = 13
- : int = 12
- : int = 20
- : int = 33
- : int = 36
- : int = 35
- : int = 34
- : int = 37
- : int = 39
- : int = 47
- : int = 46
- : int = 45
- : int = 44
*)

(* types des variables *)

param11#get_type;;
param12#get_type;;
param13#get_type;;
param14#get_type;;
param15#get_type;;
param21#get_type;;
param22#get_type;;
param23#get_type;;
param24#get_type;;
param25#get_type;;
param31#get_type;;
param32#get_type;;
param33#get_type;;
(*>>
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Symmetric_key
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Symmetric_key
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Agent
- : Types.term_type = Types.Base Types.Symmetric_key
*)

local11#get_type;;
local12#get_type;;
local13#get_type;;
local14#get_type;;
local15#get_type;;
local16#get_type;;
local21#get_type;;
local22#get_type;;
local23#get_type;;
local24#get_type;;
local25#get_type;;
local26#get_type;;
local31#get_type;;
local32#get_type;;
local33#get_type;;
local34#get_type;;
(*>>
- : Types.term_type = Types.Base Types.Nat
- : Types.term_type = Types.Base Types.Public_key
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Set [Types.Base Types.Agent]
- : Types.term_type = Types.Base Types.Nat
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Base Types.Text
- : Types.term_type = Types.Base Types.Public_key
- : Types.term_type = Types.Set [Types.Base Types.Agent]
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Channel_dy
- : Types.term_type = Types.Base Types.Channel_dy
*)


(* ***************** *)
(* Types.ident_table *)
(* Correspondance identificateur - chaine *)

Hashtbl.iter (fun id str -> print_string("\n"^(string_of_int id)^"\t"^str)) Globals.string_id#get_tab;;
(*>>
1	i
2	exp
3	xor
4	eke_Init
5	B
6	A
7	Kab
8	Rcv
9	Snd
10	State
11	Ea
12	K
13	Nb
14	Na
15	sec_k1
16	0
17	1
18	start
19	2
20	Set_20
21	secret
22	na
23	witness
24	3
25	nb
26	request
27	eke_Resp
28	A
29	B
30	Kab
31	Rcv
32	Snd
33	State
34	K
35	Nb
36	Na
37	Ea
38	sec_k2
39	Set_39
40	session
41	B
42	A
43	Kab
44	RB
45	SB
46	RA
47	SA
48	environment
49	b
50	a
51	kab
52	MGoal
53	ASGoal
54	A1Goal
55	A2Goal
56	SID
57	SID1
58	SID2
- : unit = ()
*)



(* ******************************************************************* *)
(* INSTANCIATION *)

let [inst1;inst2;inst3;inst4] = Instantiation.call_instantiation specif;;

inst1#get_session_Id;;
(*>>
- : int = 3
*)
Globals.string_id#get_name inst1#get_name;;
(*>>
- : string = "eke_Init"
*)
inst1#get_variable_instances;;
(*>>
- : int list = [3; 4; 5; 10; 13; 14; 15; 16; 11; 17]
*)
Interface.if_value_to_string Interface.atom_ground_value_to_string (Types.If_address(3));;
(*>>
- : string = "a"
*)
